
'''
This file is automatically generated; Do not edit it. :)
'''
VERSION_INFO = {
    'final': True,
    'version': '1.27.1',
    'branch_nick': 'release-1.27',
    'revision_id': '350a7e5994093846901023cf7a7aa2ec8d77e308',
    'revno': 11152
}
